<?php
//require the oss sdk
require_once __DIR__.'/sdk.class.php';
class OssAssetManager extends CAssetManager
{
    private $_oss;
    private $_baseUrl;
    private $_basePath;
    private $_published;

    public $bucket;
    public $path;
    public $host;
    public $cacheComponent = 'cache';
    public $ossAccessId;
    public $ossAccessKey;

    private function getOss(){
        if ($this->_oss === NULL)
            $this->connect();
        return $this->_oss;
    }

    public function connect()
    {
        if ( $this->ossAccessId === NULL || $this->ossAccessKey === NULL )
            throw new CException('Oss Keys are not set.');

        $this->_oss = new ALIOSS($this->ossAccessId, $this->ossAccessKey);
    }

    public function getBasePath()
    {
        if ($this->_basePath === null)
        {
            $this->_basePath = $this->path;
        }
        return $this->_basePath;
    }

    public function getBaseUrl()
    {
        if ($this->_baseUrl === null)
        {
            $this->_baseUrl = 'http://'.$this->host.'/'.$this->bucket.'/'.$this->path;
        }
        return $this->_baseUrl;
    }

    private function getCache()
    {
        if (!Yii::app()->{$this->cacheComponent})
            throw new CException('You need to configure a cache storage or set the variable cacheComponent');

        return Yii::app()->{$this->cacheComponent};
    }

    private function getCacheKey($path)
    {
        return $this->hash(Yii::app()->request->serverName).'.'.$path;
    }

    public function publish($path, $hashByName=false, $level=-1, $forceCopy=false)
    {
        if (isset($this->_published[$path]))
            return $this->_published[$path];
        else if (($src = realpath($path)) !== false)
        {
            if (is_file($src))
            {
                $dir = $this->hash($hashByName ? basename($src) : dirname($src).filemtime($src));
                $fileName = basename($src);
                $dstDir = $this->getBasePath().'/'.$dir;
                $dstFile = $dstDir.'/'.$fileName;
                if ($this->getCache()->get($this->getCacheKey($path)) === false)
                {
                    if ($this->getOss()->upload_file_by_file($this->bucket, $dstFile, $src))
                    {
                        $this->getCache()->set($this->getCacheKey($path), true, 0, new CFileCacheDependency($src));
                    }
                    else
                    {
                        throw new CException('Could not send asset do Oss');
                    }
                }

                return $this->_published[$path] = $this->getBaseUrl()."/$dir/$fileName";
            }
            else if (is_dir($src))
            {
                $dir = $this->hash($hashByName ? basename($src) : $src.filemtime($src));
                //$dstDir = $this->getBasePath().DIRECTORY_SEPARATOR.$dir;

                if ($this->getCache()->get($this->getCacheKey($path)) === false)
                {
                    $files = CFileHelper::findFiles($src, array(
                            'exclude' => $this->excludeFiles,
                            'level' => $level,
                            )
                    );

                    foreach ($files as $f)
                    {
                        $dstFile = $this->getBasePath().'/'.$dir.'/'.str_replace('\\', '/', str_replace($src.DIRECTORY_SEPARATOR, "", $f));
                        if (!$this->getOss()->upload_file_by_file($this->bucket, $dstFile, $f))
                        {
                            throw new CException('Could not send assets do Oss');
                        }
                    }

                    $this->getCache()->set($this->getCacheKey($path), true, 0, new CDirectoryCacheDependency($src));
                }


                return $this->_published[$path] = $this->getBaseUrl().'/'.$dir;
            }
        }
        throw new CException(Yii::t('yii', 'The asset "{asset}" to be published does not exist.', array('{asset}' => $path)));
    }

}

?>
